@@ -147,7 +147,6 @@
 #define SIZEOF_UINTPTR_T 4
 #define HAVE_SSIZE_T 1
 #define SIZEOF_SSIZE_T 4
-#define GC_MARK_STACKFRAME_WORD 27
 #define GETGROUPS_T gid_t
 #define RETSIGTYPE void
 #define HAVE_ALLOCA 1
@@ -279,6 +278,7 @@
 #define USE_ELF 1
 #define DLEXT_MAXLEN 3
 #define DLEXT ".so"
+#define LIBDIR_BASENAME "lib"
 #define HAVE_PTHREAD_H 1
 #define RUBY_LIB_VERSION_STYLE 3	/* full */
 #define RUBY_EXEC_PREFIX "/usr/local"
