@@ -2,12 +2,12 @@
 # ipaddr.rb - A class to manipulate an IP address
 #
 # Copyright (c) 2002 Hajimu UMEMOTO <ume@mahoroba.org>.
-# Copyright (c) 2007 Akinori MUSHA <knu@iDaemons.org>.
+# Copyright (c) 2007, 2009, 2012 Akinori MUSHA <knu@iDaemons.org>.
 # All rights reserved.
 #
 # You can redistribute and/or modify it under the same terms as Ruby.
 #
-# $Id: ipaddr.rb 27000 2010-03-21 12:10:53Z akr $
+# $Id: ipaddr.rb 35917 2012-06-05 05:04:19Z knu $
 #
 # Contact:
 #   - Akinori MUSHA <knu@iDaemons.org> (current maintainer)
@@ -17,49 +17,6 @@
 #
 require 'socket'
 
-unless Socket.const_defined? "AF_INET6"
-  class Socket < BasicSocket
-    AF_INET6 = Object.new
-  end
-
-  class << IPSocket
-    def valid_v4?(addr)
-      if /\A(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})\Z/ =~ addr
-        return $~.captures.all? {|i| i.to_i < 256}
-      end
-      return false
-    end
-
-    def valid_v6?(addr)
-      # IPv6 (normal)
-      return true if /\A[\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*\Z/ =~ addr
-      return true if /\A[\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*::([\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*)?\Z/ =~ addr
-      return true if /\A::([\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*)?\Z/ =~ addr
-      # IPv6 (IPv4 compat)
-      return true if /\A[\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*:/ =~ addr && valid_v4?($')
-      return true if /\A[\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*::([\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*:)?/ =~ addr && valid_v4?($')
-      return true if /\A::([\dA-Fa-f]{1,4}(:[\dA-Fa-f]{1,4})*:)?/ =~ addr && valid_v4?($')
-
-      false
-    end
-
-    def valid?(addr)
-      valid_v4?(addr) || valid_v6?(addr)
-    end
-
-    alias getaddress_orig getaddress
-    def getaddress(s)
-      if valid?(s)
-        s
-      elsif /\A[-A-Za-z\d.]+\Z/ =~ s
-        getaddress_orig(s)
-      else
-        raise ArgumentError, "invalid address"
-      end
-    end
-  end
-end
-
 # IPAddr provides a set of methods to manipulate an IP address.  Both IPv4 and
 # IPv6 are supported.
 #
@@ -69,24 +26,62 @@ end
 #
 #   ipaddr1 = IPAddr.new "3ffe:505:2::1"
 #
-#   p ipaddr1			#=> #<IPAddr: IPv6:3ffe:0505:0002:0000:0000:0000:0000:0001/ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff>
+#   p ipaddr1                   #=> #<IPAddr: IPv6:3ffe:0505:0002:0000:0000:0000:0000:0001/ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff>
 #
-#   p ipaddr1.to_s		#=> "3ffe:505:2::1"
+#   p ipaddr1.to_s              #=> "3ffe:505:2::1"
 #
-#   ipaddr2 = ipaddr1.mask(48)	#=> #<IPAddr: IPv6:3ffe:0505:0002:0000:0000:0000:0000:0000/ffff:ffff:ffff:0000:0000:0000:0000:0000>
+#   ipaddr2 = ipaddr1.mask(48)  #=> #<IPAddr: IPv6:3ffe:0505:0002:0000:0000:0000:0000:0000/ffff:ffff:ffff:0000:0000:0000:0000:0000>
 #
-#   p ipaddr2.to_s		#=> "3ffe:505:2::"
+#   p ipaddr2.to_s              #=> "3ffe:505:2::"
 #
 #   ipaddr3 = IPAddr.new "192.168.2.0/24"
 #
-#   p ipaddr3			#=> #<IPAddr: IPv4:192.168.2.0/255.255.255.0>
+#   p ipaddr3                   #=> #<IPAddr: IPv4:192.168.2.0/255.255.255.0>
 
 class IPAddr
 
+  # 32 bit mask for IPv4
   IN4MASK = 0xffffffff
+  # 128 bit mask for IPv4
   IN6MASK = 0xffffffffffffffffffffffffffffffff
+  # Format string for IPv6
   IN6FORMAT = (["%.4x"] * 8).join(':')
 
+  # Regexp _internally_ used for parsing IPv4 address.
+  RE_IPV4ADDRLIKE = %r{
+    \A
+    (\d+) \. (\d+) \. (\d+) \. (\d+)
+    \z
+  }x
+
+  # Regexp _internally_ used for parsing IPv6 address.
+  RE_IPV6ADDRLIKE_FULL = %r{
+    \A
+    (?:
+      (?: [\da-f]{1,4} : ){7} [\da-f]{1,4}
+    |
+      ( (?: [\da-f]{1,4} : ){6} )
+      (\d+) \. (\d+) \. (\d+) \. (\d+)
+    )
+    \z
+  }xi
+
+  # Regexp _internally_ used for parsing IPv6 address.
+  RE_IPV6ADDRLIKE_COMPRESSED = %r{
+    \A
+    ( (?: (?: [\da-f]{1,4} : )* [\da-f]{1,4} )? )
+    ::
+    ( (?:
+      ( (?: [\da-f]{1,4} : )* )
+      (?:
+        [\da-f]{1,4}
+      |
+        (\d+) \. (\d+) \. (\d+) \. (\d+)
+      )
+    )? )
+    \z
+  }xi
+
   # Returns the address family of this IP address.
   attr_reader :family
 
@@ -154,13 +149,13 @@ class IPAddr
   #   net1 = IPAddr.new("192.168.2.0/24")
   #   net2 = IPAddr.new("192.168.2.100")
   #   net3 = IPAddr.new("192.168.3.0")
-  #   p net1.include?(net2)	#=> true
-  #   p net1.include?(net3)	#=> false
+  #   p net1.include?(net2)     #=> true
+  #   p net1.include?(net3)     #=> false
   def include?(other)
     other = coerce_other(other)
     if ipv4_mapped?
       if (@mask_addr >> 32) != 0xffffffffffffffffffffffff
-	return false
+        return false
       end
       mask_addr = (@mask_addr & IN4MASK)
       addr = (@addr & IN4MASK)
@@ -197,7 +192,7 @@ class IPAddr
 
     str.gsub!(/\b0{1,3}([\da-f]+)\b/i, '\1')
     loop do
-      break if str.sub!(/\A0:0:0:0:0:0:0:0\Z/, '::')
+      break if str.sub!(/\A0:0:0:0:0:0:0:0\z/, '::')
       break if str.sub!(/\b0:0:0:0:0:0:0\b/, ':')
       break if str.sub!(/\b0:0:0:0:0:0\b/, ':')
       break if str.sub!(/\b0:0:0:0:0\b/, ':')
@@ -208,7 +203,7 @@ class IPAddr
     end
     str.sub!(/:{3,}/, '::')
 
-    if /\A::(ffff:)?([\da-f]{1,4}):([\da-f]{1,4})\Z/i =~ str
+    if /\A::(ffff:)?([\da-f]{1,4}):([\da-f]{1,4})\z/i =~ str
       str = sprintf('::%s%d.%d.%d.%d', $1, $2.hex / 256, $2.hex % 256, $3.hex / 256, $3.hex % 256)
     end
 
@@ -228,7 +223,7 @@ class IPAddr
       return [@addr].pack('N')
     when Socket::AF_INET6
       return (0..7).map { |i|
-	(@addr >> (112 - 16 * i)) & 0xffff
+        (@addr >> (112 - 16 * i)) & 0xffff
       }.pack('n8')
     else
       raise "unsupported address family"
@@ -369,20 +364,23 @@ class IPAddr
       raise "unsupported address family"
     end
     return sprintf("#<%s: %s:%s/%s>", self.class.name,
-		   af, _to_string(@addr), _to_string(@mask_addr))
+                   af, _to_string(@addr), _to_string(@mask_addr))
   end
 
   protected
 
+  # Set +@addr+, the internal stored ip address, to given +addr+. The
+  # parameter +addr+ is validated using the first +family+ member,
+  # which is +Socket::AF_INET+ or +Socket::AF_INET6+.
   def set(addr, *family)
     case family[0] ? family[0] : @family
     when Socket::AF_INET
       if addr < 0 || addr > IN4MASK
-	raise ArgumentError, "invalid address"
+        raise ArgumentError, "invalid address"
       end
     when Socket::AF_INET6
       if addr < 0 || addr > IN6MASK
-	raise ArgumentError, "invalid address"
+        raise ArgumentError, "invalid address"
       end
     else
       raise ArgumentError, "unsupported address family"
@@ -394,18 +392,19 @@ class IPAddr
     return self
   end
 
+  # Set current netmask to given mask. 
   def mask!(mask)
     if mask.kind_of?(String)
       if mask =~ /^\d+$/
-	prefixlen = mask.to_i
+        prefixlen = mask.to_i
       else
-	m = IPAddr.new(mask)
-	if m.family != @family
-	  raise ArgumentError, "address family is not same"
-	end
-	@mask_addr = m.to_i
-	@addr &= @mask_addr
-	return self
+        m = IPAddr.new(mask)
+        if m.family != @family
+          raise ArgumentError, "address family is not same"
+        end
+        @mask_addr = m.to_i
+        @addr &= @mask_addr
+        return self
       end
     else
       prefixlen = mask
@@ -413,13 +412,13 @@ class IPAddr
     case @family
     when Socket::AF_INET
       if prefixlen < 0 || prefixlen > 32
-	raise ArgumentError, "invalid length"
+        raise ArgumentError, "invalid length"
       end
       masklen = 32 - prefixlen
       @mask_addr = ((IN4MASK >> masklen) << masklen)
     when Socket::AF_INET6
       if prefixlen < 0 || prefixlen > 128
-	raise ArgumentError, "invalid length"
+        raise ArgumentError, "invalid length"
       end
       masklen = 128 - prefixlen
       @mask_addr = ((IN6MASK >> masklen) << masklen)
@@ -458,9 +457,9 @@ class IPAddr
         @mask_addr = (family == Socket::AF_INET) ? IN4MASK : IN6MASK
         return
       when Socket::AF_UNSPEC
-	raise ArgumentError, "address family must be specified"
+        raise ArgumentError, "address family must be specified"
       else
-	raise ArgumentError, "unsupported address family: #{family}"
+        raise ArgumentError, "unsupported address family: #{family}"
       end
     end
     prefix, prefixlen = addr.split('/')
@@ -470,17 +469,12 @@ class IPAddr
     end
     # It seems AI_NUMERICHOST doesn't do the job.
     #Socket.getaddrinfo(left, nil, Socket::AF_INET6, Socket::SOCK_STREAM, nil,
-    #		       Socket::AI_NUMERICHOST)
-    begin
-      IPSocket.getaddress(prefix)		# test if address is valid
-    rescue
-      raise ArgumentError, "invalid address"
-    end
+    #                  Socket::AI_NUMERICHOST)
     @addr = @family = nil
     if family == Socket::AF_UNSPEC || family == Socket::AF_INET
       @addr = in_addr(prefix)
       if @addr
-	@family = Socket::AF_INET
+        @family = Socket::AF_INET
       end
     end
     if !@addr && (family == Socket::AF_UNSPEC || family == Socket::AF_INET6)
@@ -509,26 +503,44 @@ class IPAddr
   end
 
   def in_addr(addr)
-    if addr =~ /^\d+\.\d+\.\d+\.\d+$/
-      return addr.split('.').inject(0) { |i, s|
-        i << 8 | s.to_i
-      }
+    case addr
+    when Array
+      octets = addr
+    else
+      m = RE_IPV4ADDRLIKE.match(addr) or return nil
+      octets = m.captures
     end
-    return nil
+    octets.inject(0) { |i, s|
+      (n = s.to_i) < 256 or raise ArgumentError, "invalid address"
+      s.match(/\A0./) and raise ArgumentError, "zero-filled number is ambiguous"
+      i << 8 | n
+    }
   end
 
   def in6_addr(left)
     case left
-    when /^::ffff:(\d+\.\d+\.\d+\.\d+)$/i
-      return in_addr($1) + 0xffff00000000
-    when /^::(\d+\.\d+\.\d+\.\d+)$/i
-      return in_addr($1)
-    when /[^0-9a-f:]/i
-      raise ArgumentError, "invalid address"
-    when /^(.*)::(.*)$/
-      left, right = $1, $2
-    else
+    when RE_IPV6ADDRLIKE_FULL
+      if $2
+        addr = in_addr($~[2,4])
+        left = $1 + ':'
+      else
+        addr = 0
+      end
       right = ''
+    when RE_IPV6ADDRLIKE_COMPRESSED
+      if $4
+        left.count(':') <= 6 or raise ArgumentError, "invalid address"
+        addr = in_addr($~[4,4])
+        left = $1
+        right = $3 + '0:0'
+      else
+        left.count(':') <= 7 or raise ArgumentError, "invalid address"
+        left = $1
+        right = $2
+        addr = 0
+      end
+    else
+      raise ArgumentError, "invalid address"
     end
     l = left.split(':')
     r = right.split(':')
@@ -536,9 +548,9 @@ class IPAddr
     if rest < 0
       return nil
     end
-    return (l + Array.new(rest, '0') + r).inject(0) { |i, s|
+    (l + Array.new(rest, '0') + r).inject(0) { |i, s|
       i << 16 | s.hex
-    }
+    } | addr
   end
 
   def addr_mask(addr)
@@ -556,7 +568,7 @@ class IPAddr
     case @family
     when Socket::AF_INET
       return (0..3).map { |i|
-	(@addr >> (8 * i)) & 0xff
+        (@addr >> (8 * i)) & 0xff
       }.join('.')
     when Socket::AF_INET6
       return ("%.32x" % @addr).reverse!.gsub!(/.(?!$)/, '\&.')
@@ -569,7 +581,7 @@ class IPAddr
     case @family
     when Socket::AF_INET
       return (0..3).map { |i|
-	(addr >> (24 - 8 * i)) & 0xff
+        (addr >> (24 - 8 * i)) & 0xff
       }.join('.')
     when Socket::AF_INET6
       return (("%.32x" % addr).gsub!(/.{4}(?!$)/, '\&:'))
@@ -580,6 +592,55 @@ class IPAddr
 
 end
 
+unless Socket.const_defined? "AF_INET6"
+  class Socket < BasicSocket
+    # IPv6 protocol family
+    AF_INET6 = Object.new
+  end
+
+  class << IPSocket
+    private
+
+    def valid_v6?(addr)
+      case addr
+      when IPAddr::RE_IPV6ADDRLIKE_FULL
+        if $2
+          $~[2,4].all? {|i| i.to_i < 256 }
+        else
+          true
+        end
+      when IPAddr::RE_IPV6ADDRLIKE_COMPRESSED
+        if $4
+          addr.count(':') <= 6 && $~[4,4].all? {|i| i.to_i < 256}
+        else
+          addr.count(':') <= 7
+        end
+      else
+        false
+      end
+    end
+
+    alias getaddress_orig getaddress
+
+    public
+
+    # Returns a +String+ based representation of a valid DNS hostname,
+    # IPv4 or IPv6 address.
+    # 
+    #   IPSocket.getaddress 'localhost'         #=> "::1"
+    #   IPSocket.getaddress 'broadcasthost'     #=> "255.255.255.255"
+    #   IPSocket.getaddress 'www.ruby-lang.org' #=> "221.186.184.68"
+    #   IPSocket.getaddress 'www.ccc.de'        #=> "2a00:1328:e102:ccc0::122"
+    def getaddress(s)
+      if valid_v6?(s)
+        s
+      else
+        getaddress_orig(s)
+      end
+    end
+  end
+end
+
 if $0 == __FILE__
   eval DATA.read, nil, $0, __LINE__+4
 end
@@ -590,10 +651,15 @@ require 'test/unit'
 
 class TC_IPAddr < Test::Unit::TestCase
   def test_s_new
-    assert_nothing_raised {
-      IPAddr.new("3FFE:505:ffff::/48")
-      IPAddr.new("0:0:0:1::")
-      IPAddr.new("2001:200:300::/48")
+    [
+      ["3FFE:505:ffff::/48"],
+      ["0:0:0:1::"],
+      ["2001:200:300::/48"],
+      ["2001:200:300::192.168.1.2/48"],
+    ].each { |args|
+      assert_nothing_raised {
+        IPAddr.new(*args)
+      }
     }
 
     a = IPAddr.new
@@ -648,15 +714,16 @@ class TC_IPAddr < Test::Unit::TestCase
     assert_equal("2001:200:300::", IPAddr.new("[2001:200:300::]/48").to_s)
 
     [
+      ["192.168.0.256"],
+      ["192.168.0.011"],
       ["fe80::1%fxp0"],
       ["::1/255.255.255.0"],
-      ["::1:192.168.1.2/120"],
       [IPAddr.new("::1").to_i],
       ["::ffff:192.168.1.2/120", Socket::AF_INET],
       ["[192.168.1.2]/120"],
     ].each { |args|
       assert_raises(ArgumentError) {
-	IPAddr.new(*args)
+        IPAddr.new(*args)
       }
     }
   end
@@ -757,7 +824,7 @@ class TC_Operator < Test::Unit::TestCase
     assert_equal("3ffe:505:2:1::", a.to_s)
     assert_equal("3ffe:505:2::", @a.to_s)
     assert_equal("3ffe:505:2:1::",
-		 (@a | 0x00000000000000010000000000000000).to_s)
+                 (@a | 0x00000000000000010000000000000000).to_s)
   end
 
   def test_and
@@ -792,7 +859,9 @@ class TC_Operator < Test::Unit::TestCase
   end
 
   def test_equal
-    assert_equal(true, @a == IPAddr.new("3ffe:505:2::"))
+    assert_equal(true, @a == IPAddr.new("3FFE:505:2::"))
+    assert_equal(true, @a == IPAddr.new("3ffe:0505:0002::"))
+    assert_equal(true, @a == IPAddr.new("3ffe:0505:0002:0:0:0:0:0"))
     assert_equal(false, @a == IPAddr.new("3ffe:505:3::"))
     assert_equal(true, @a != IPAddr.new("3ffe:505:3::"))
     assert_equal(false, @a != IPAddr.new("3ffe:505:2::"))
