@@ -13,7 +13,7 @@ module RbConfig
   CONFIG["MAJOR"] = "1"
   CONFIG["MINOR"] = "9"
   CONFIG["TEENY"] = "1"
-  CONFIG["PATCHLEVEL"] = "194"
+  CONFIG["PATCHLEVEL"] = "392"
   CONFIG["INSTALL"] = '/usr/bin/install -c -o root -g bin'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
@@ -42,6 +42,7 @@ module RbConfig
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
   CONFIG["rubylibprefix"] = "$(libdir)/$(RUBY_BASE_NAME)"
   CONFIG["MAKEFILES"] = "Makefile"
+  CONFIG["PLATFORM_DIR"] = ""
   CONFIG["THREAD_MODEL"] = "pthread"
   CONFIG["SYMBOL_PREFIX"] = ""
   CONFIG["EXPORT_PREFIX"] = ""
@@ -63,7 +64,7 @@ module RbConfig
   CONFIG["rubyw_install_name"] = ""
   CONFIG["LIBRUBY_DLDFLAGS"] = ""
   CONFIG["LIBRUBY_LDSHARED"] = "$(CC) -shared $(CCDLFLAGS)"
-  CONFIG["warnflags"] = "-Wextra -Wno-unused-parameter -Wno-parentheses -Wno-long-long -Wno-missing-field-initializers -Werror=pointer-arith -Werror=write-strings -Werror=declaration-after-statement -Werror=implicit-function-declaration"
+  CONFIG["warnflags"] = "-Wall -Wextra -Wno-unused-parameter -Wno-parentheses -Wno-long-long -Wno-missing-field-initializers -Werror=pointer-arith -Werror=write-strings -Werror=declaration-after-statement -Werror=implicit-function-declaration"
   CONFIG["debugflags"] = "-ggdb"
   CONFIG["optflags"] = "-O3"
   CONFIG["cxxflags"] = " $(optflags) $(debugflags) $(warnflags)"
@@ -117,7 +118,7 @@ module RbConfig
   CONFIG["NM"] = "nm"
   CONFIG["DLLWRAP"] = ""
   CONFIG["WINDRES"] = ""
-  CONFIG["OBJCOPY"] = "objcopy"
+  CONFIG["OBJCOPY"] = ":"
   CONFIG["OBJDUMP"] = "objdump"
   CONFIG["ASFLAGS"] = ""
   CONFIG["AS"] = "as"
@@ -139,21 +140,21 @@ module RbConfig
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -Wl,-E"
   CONFIG["CFLAGS"] = "-O2 -pipe -fPIC"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd5.2"
+  CONFIG["target_os"] = "openbsd5.3"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "sh"
-  CONFIG["target"] = "sh-unknown-openbsd5.2"
-  CONFIG["host_os"] = "openbsd5.2"
+  CONFIG["target"] = "sh-unknown-openbsd5.3"
+  CONFIG["host_os"] = "openbsd5.3"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "sh"
-  CONFIG["host"] = "sh-unknown-openbsd5.2"
+  CONFIG["host"] = "sh-unknown-openbsd5.3"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd5.2"
+  CONFIG["build_os"] = "openbsd5.3"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "sh"
-  CONFIG["build"] = "sh-unknown-openbsd5.2"
-  CONFIG["RUBY_RELEASE_DATE"] = "2012-04-20"
+  CONFIG["build"] = "sh-unknown-openbsd5.3"
+  CONFIG["RUBY_RELEASE_DATE"] = "2013-02-22"
   CONFIG["RUBY_PROGRAM_VERSION"] = "1.9.3"
   CONFIG["BASERUBY"] = "echo executable host ruby is required.  use --with-baseruby option.; false"
   CONFIG["target_alias"] = ""
